import logging

from aj.plugins.core.api.tasks import Task
from aj.plugins.packages.api import PackageManager


class UpdateLists(Task):
    name = 'Update package lists'

    def __init__(self, context, manager_id=None):
        Task.__init__(self, context)
        for mgr in PackageManager.all(self.context):
            if mgr.id == manager_id:
                self.manager = mgr
                break
        else:
            logging.error('Package manager %s not found', manager_id)

    def run(self):
        self.manager.update_lists(self.report_progress)
        self.push('packages', 'refresh')



try:
    import xmlrpclib
except ImportError:
    import xmlrpc.client as xmlrpclib

import pip

from aj.api import *
from aj.plugins.packages.api import PackageManager, Package


@component(PackageManager)
class PIPPackageManager(PackageManager):
    id = 'pip'
    name = 'PIP'

    def __init__(self, context):
        PackageManager.__init__(self, context)
        self.client = xmlrpclib.ServerProxy('https://pypi.python.org/pypi')

    def __make_package_pipdist(self, dist):
        p = Package(self)
        p.id = '%s==%s' % (dist.key, dist.version)
        p.name = dist.key
        p.version = dist.version
        p.description = None
        p.is_installed = True
        p.installed_version = dist.version
        return p

    def __make_package(self, dist):
        p = Package(self)
        p.id = '%s==%s' % (dist['name'], dist['version'])
        p.name = dist['name']
        p.version = dist['version']
        p.description = dist['summary']
        for d in pip.get_installed_distributions():
            if d.key == p.name:
                p.is_installed = True
                p.installed_version = d.version
                p.is_upgradeable = p.installed_version != p.version
        return p

    def list(self, query=None):
        for dist in self.client.search({'name': query}):
            yield self.__make_package(dist)

    def get(self, _id):
        for d in pip.get_installed_distributions():
            if d.key == _id.split('==')[0]:
                return self.__make_package_pipdist(d)

    def update_lists(self, progress_callback):
        pass

    def get_apply_cmd(self, selection):
        cmd = 'pip install '
        for sel in selection:
            if sel['operation'] in ['install', 'upgrade']:
                cmd += sel['package']['id']
        cmd += '; pip uninstall '
        for sel in selection:
            if sel['operation'] in ['remove']:
                cmd += sel['package']['name']
        return cmd



from aj.api import *


class Package(object):
    def __init__(self, manager):
        self.manager = manager
        self.id = None
        self.name = None
        self.version = None
        self.description = None
        self.is_installed = None
        self.is_upgradeable = None
        self.installed_version = None


@interface
class PackageManager(object):
    id = None
    name = None
    update_command = None

    def __init__(self, context):
        self.context = context

    def list(self, query=None):
        raise NotImplementedError

    def get(self, _id):
        raise NotImplementedError

    def update_lists(self, progress_callback):
        raise NotImplementedError

    def get_apply_cmd(self, selection):
        raise NotImplementedError



import api
import main
import views
import tasks

try:
    import managers.apt_manager
except ImportError:
    pass

try:
    import managers.yum_manager
except ImportError:
    pass

import managers.pip_manager



from aj.plugins.core.api.sidebar import *

from aj.plugins.packages.api import PackageManager


@component(SidebarItemProvider)
class ItemProvider(SidebarItemProvider):
    def __init__(self, context):
        self.context = context

    def provide(self):
        children = [{
            'attach': 'packages',
            'name': mgr.name,
            'icon': 'gift',
            'url': '/view/packages/%s' % mgr.id,
            'children': [],
        } for mgr in PackageManager.all(self.context)]
        return [
            {
                'attach': 'category:system',
                'id': 'packages',
                'name': 'Packages',
                'icon': 'gift',
                'url': '/view/packages/%s' % PackageManager.all(self.context)[0].id,
                'children': children,
            }
        ]
